//
//  LKBitmapContext.m
//  Vectors
//
//  Created by Károly Lőrentey on 2010. 5. 19.
//  Copyright 2010 Károly Lőrentey. All rights reserved.
//

#import "LKBitmapContext.h"

@implementation LKBitmapContext

- (id) init {
    abort();
}

- (id)initWithWidth:(size_t)width height:(size_t)height alpha:(BOOL)alpha {
    assert(width > 0 && height > 0);

    struct context_params {
        CGColorSpaceRef colorspace;
        CGBitmapInfo bi;
        size_t bpc;
        float scale;
    };
    static struct context_params alphaParams, opaqueParams;
    struct context_params *params = (alpha ? &alphaParams : &opaqueParams);
    if (params->colorspace == NULL) {
        UIGraphicsBeginImageContextWithOptions(CGSizeMake(10, 10), !alpha, 0.0f);
        CGContextRef c = UIGraphicsGetCurrentContext();
        params->colorspace = CGBitmapContextGetColorSpace(c);
        CGColorSpaceRetain(params->colorspace);
        params->bi = CGBitmapContextGetBitmapInfo(c) | CGBitmapContextGetAlphaInfo(c);
        params->bpc = CGBitmapContextGetBitsPerComponent(c);
        
        size_t h = CGBitmapContextGetHeight(c);
        params->scale = (float)h / (float)10;
        
        UIGraphicsEndImageContext();        
    }
    
    if ((self = [super init])) {
        _context = CGBitmapContextCreate(NULL, params->scale * width, params->scale * height, 
                                         params->bpc, 0, params->colorspace, params->bi);
        if (_context == NULL) {
            LKError(@"error creating bitmap context");
            abort();
        }
        //LKDebug(@"Scale is %f", params->scale);
        CGContextScaleCTM(_context, params->scale, params->scale);
        
        // Flip coordinate system such that the origin is at the upper left corner.
        CGContextTranslateCTM (_context, 0, height);
        CGContextScaleCTM (_context, 1.0, -1.0);
    }
    return self;
}

- (void)dealloc {
    if (_context != nil) {
        CGContextRelease(_context);
        _context = nil;
    }
    [super dealloc];
}

- (size_t)width {
    return CGBitmapContextGetWidth(_context);
}

- (size_t)height {
    return CGBitmapContextGetHeight(_context);
}

- (CGContextRef)CGContext {
    return _context;
}

- (UIImage *)image {
    CGImageRef cgimage = CGBitmapContextCreateImage(_context);
    UIImage *image = [UIImage imageWithCGImage:cgimage];
    CGImageRelease(cgimage);
    return image;
}
@end
