//
//  LKFloat.h
//  LKCommon
//
//  Created by Károly Lőrentey on 2011-02-02.
//  Copyright 2011 Károly Lőrentey. All rights reserved.
//

#ifndef _LK_FLOAT_H_INCLUDED
#define _LK_FLOAT_H_INCLUDED

#include <stdbool.h>
#include <math.h>

// Clamp VAL between MIN and MAX, inclusive.
static inline int 
clamp(int val, int min, int max)
{
    return MAX(MIN(val, max), min);
}

// Macro version of clamp for use in constant initializers.
#define clamp_m(val, min, max) MAX(MIN(val, max), min)

// Clamp VAL between MIN and MAX, inclusive.
static inline double 
fclamp(double val, double min, double max)
{
    return fmax(fmin(val, max), min);
}

// Clamp VAL between MIN and MAX, inclusive.
static inline float
fclampf(float val, float min, float max)
{
    return fmaxf(fminf(val, max), min);
}

// Floating point comparison with absolute allowable error.
static inline BOOL 
fequalf_abs(float a, float b, float epsilon)
{
    return fabsf(a - b) < epsilon;
}

// Floating point comparison with relative allowable error (approximated using Units in the Last Place)
extern BOOL fequalf_ulps(float a, float b, int max_ulps);
#endif