//
//  LKFloat.m
//  LKCommon
//
//  Created by Károly Lőrentey on 2011-02-02.
//  Copyright 2011 Károly Lőrentey. All rights reserved.
//

#include <Foundation/Foundation.h>
#include "LKFloat.h"

BOOL
fequalf_ulps(float a, float b, int max_ulps)
{
    // Adapted from http://www.cygnus-software.com/papers/comparingfloats/comparingfloats.htm
    
    if (isnan(a) || isnan(b))
        return NO;
    
    // Make sure maxUlps is non-negative and small enough that the
    // default NAN won't compare as equal to anything.
    assert(max_ulps > 0 && max_ulps < 4 * 1024 * 1024);
    
    union { 
        int32_t i;
        float f;
    } au, bu;
    
    au.f = a;
    bu.f = b;

    // Make negative values ordered right.
    if (au.i < 0)
        au.i = 0x80000000 - au.i;
    if (bu.i < 0)
        bu.i = 0x80000000 - bu.i;
    
    int intDiff = abs(au.i - bu.i);
    return intDiff <= max_ulps;
}