//
//  LKTiledScrollView.h
//  Vectors
//
//  Created by Károly Lőrentey on 2010-03-08.
//  Copyright 2010 Károly Lőrentey. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <QuartzCore/QuartzCore.h>

@protocol LKTiledCanvasViewDelegate;

@interface LKTiledCanvasView : UIView<UIGestureRecognizerDelegate> {
@private
    id <LKTiledCanvasViewDelegate> _delegate;

    CAScrollLayer *_tileCollectorLayer;

    CGSize _rszTile;
    id _defaultContents;

    NSMutableSet *_reusableTiles;
    NSMutableArray *_tiles;

    int _firstVisibleRow;
    int _firstVisibleColumn;
    int _lastVisibleRow;
    int _lastVisibleColumn;

    CGRect _rcvLayout;
    CGRect _rcdVisible;
    CGRect _rcdContents;

    float _minZoomScale;
    float _maxZoomScale;
    float _appliedZoomScale;
    float _fastZoomScale;
    
#ifdef DEBUG
    BOOL _enableDebugOutput;
#endif
    float _scaleOrig;
}

@property (nonatomic, assign) IBOutlet id <LKTiledCanvasViewDelegate> delegate;
@property (nonatomic, assign) CGSize tileSize;
@property (nonatomic, assign, readonly) float zoomScale; // Current scale
@property (nonatomic, assign) float minimumZoomScale;
@property (nonatomic, assign) float maximumZoomScale;
#ifdef DEBUG
@property (nonatomic, assign) BOOL enableDebugOutput;
#endif

@property (nonatomic, assign) CGRect rcdContents; // The document's bounding rectangle.

// Coordinate conversion methods.
// pt: CGPoint; rs: CGSize; rc: CGRect
// v: view coordinate system
// d: document coordinate system
- (CGPoint)ptdFromPtv:(CGPoint)ptv;
- (CGPoint)ptvFromPtd:(CGPoint)ptd;

- (CGSize)rsdFromRsv:(CGSize)rsv;
- (CGSize)rsvFromRsd:(CGSize)rsd;

- (CGRect)rcdFromRcv:(CGRect)rcv;
- (CGRect)rcvFromRcd:(CGRect)rcd;

// Center of current viewport in document coordinates.
- (CGPoint)ptdCenter; 
// Current viewport rectangle in document coordinates.
- (CGRect)rcdVisible; 

// Recreate and redraw all tiles.
- (void)reloadData;

// Immediately redraw a portion of the document.
- (void)setNeedsDisplayInRcd:(CGRect)rcd;

// Move position so that ptd is at the origin.
- (void)translateToPoint:(CGPoint)ptd;

// Move position by the given vector.
- (void)translateByVector:(CGSize)vkd;

// Position and zoom the view so that rcd is centered in it and zoomed to fit exactly.
// If fast is YES, use hardware scaling when possible (the canvas will automatically redraw at final scale later).
- (void)zoomToRcd:(CGRect)rcd fast:(BOOL)fast;

// Zoom to the given scale centered at the given point.
- (void)zoomToScale:(float)scale aroundPtd:(CGPoint)ptd fast:(BOOL)fast;

// Immediately redraw at current scale, canceling any hw scale in effect. Does nothing if not currently scaled.
- (void)commitFastZoom;

@end


@protocol LKTiledCanvasViewDelegate <NSObject>
// Called to draw a single tile. You can get the bounds of the tile using CGContextGetClipBoundingBox.
- (void)drawTileForView:(LKTiledCanvasView *)canvasView
              inContext:(CGContextRef)context;
@end

